"""Objects used in the Apibara client."""

from dataclasses import dataclass
from datetime import datetime
from typing import List, Optional, Union

import apibara.application.indexer_service_pb2 as indexer_service_pb2
from apibara.starknet import get_selector_from_name


@dataclass
class Topic:
    choices: List[bytes]

    @staticmethod
    def from_proto(p: indexer_service_pb2.Topic):
        return Topic([c.value for c in p.choices])

    def to_proto(self) -> indexer_service_pb2.Topic:
        return indexer_service_pb2.Topic(
            choices=[indexer_service_pb2.TopicValue(value=c) for c in self.choices]
        )

    def __str__(self) -> str:
        choices = [f"0x{c.hex()}" for c in self.choices]
        return f'Topic({"|".join(choices)})'


@dataclass
class EventFilter:
    name: Optional[str]
    address: Optional[bytes]
    topics: List[Topic]

    @classmethod
    def from_event_name(
        cls, name: str, address: Optional[Union[str, bytes]]
    ) -> "EventFilter":
        """Create an EventFilter from the event name."""
        if isinstance(address, str):
            address = bytes.fromhex(address.replace("0x", ""))
        topic_hex = hex(get_selector_from_name(name))[2:]
        # fromhex requires an even number of digits
        if len(topic_hex) % 2 == 1:
            topic_hex = "0" + topic_hex
        topic_value = bytes.fromhex(topic_hex)
        topics = [Topic([topic_value])]

        return cls(name, address, topics)

    @staticmethod
    def from_proto(p: indexer_service_pb2.EventFilter):
        address = None
        if len(p.address) > 0:
            address = p.address

        topics = [Topic.from_proto(t) for t in p.topics]
        return EventFilter(name=None, address=address, topics=topics)

    def to_proto(self) -> indexer_service_pb2.EventFilter:
        return indexer_service_pb2.EventFilter(
            address=self.address, topics=[t.to_proto() for t in self.topics]
        )

    def __str__(self) -> str:
        address = ""
        if self.address is not None:
            address = f"address=0x{self.address.hex()}, "
        return (
            f'EventFilter({address}topics=[{", ".join(str(t) for t in self.topics)}])'
        )


@dataclass
class Indexer:
    id: str
    index_from_block: int
    indexed_to_block: int
    filters: List[EventFilter]

    @staticmethod
    def from_proto(p: indexer_service_pb2.Indexer):
        filters = [EventFilter.from_proto(f) for f in p.filters]
        return Indexer(p.id, p.index_from_block, p.indexed_to_block, filters)

    def __str__(self) -> str:
        return f'Indexer(id={self.id}, blocks=[{self.index_from_block}, {self.indexed_to_block}], filters=[{", ".join(str(f) for f in self.filters)}])'


@dataclass
class IndexerConnected:
    indexer: Indexer

    @staticmethod
    def from_proto(p: indexer_service_pb2.IndexerConnected):
        app = Indexer.from_proto(p.indexer)
        return IndexerConnected(app)


@dataclass
class BlockHeader:
    hash: bytes
    parent_hash: Optional[bytes]
    number: int
    timestamp: datetime

    @staticmethod
    def from_proto(p: indexer_service_pb2.BlockHeader):
        dt = datetime.fromtimestamp(p.timestamp.seconds)
        return BlockHeader(bytes(p.hash), bytes(p.parent_hash), p.number, dt)

    def __str__(self) -> str:
        return f"BlockHeader(hash=0x{self.hash.hex()}, parent_hash=0x{self.parent_hash.hex()}, number={self.number}, timestamp={self.timestamp})"


@dataclass
class Event:
    name: Optional[str]
    address: bytes
    block_index: int
    topics: List[bytes]
    data: List[bytes]

    @staticmethod
    def from_proto(p: indexer_service_pb2.Event):
        topics = [bytes(t.value) for t in p.topics]
        data = [bytes(d.value) for d in p.data]
        return Event(None, bytes(p.address), p.block_index, topics, data)

    def __str__(self) -> str:
        name = ""
        if self.name is not None:
            name = f"name={self.name}, "
        return f"Event({name}address=0x{self.address.hex()}, block_index={self.block_index}, ...{len(self.topics)} topics, ...{len(self.data)} data)"


@dataclass
class NewBlock:
    new_head: BlockHeader

    @staticmethod
    def from_proto(p: indexer_service_pb2.NewBlock):
        new_head = BlockHeader.from_proto(p.new_head)
        return NewBlock(new_head)

    def __str__(self) -> str:
        return f"NewBlock(new_head={self.new_head})"


@dataclass
class Reorg:
    new_head: BlockHeader

    @staticmethod
    def from_proto(p: indexer_service_pb2.Reorg):
        new_head = BlockHeader.from_proto(p.new_head)
        return Reorg(new_head)

    def __str__(self) -> str:
        return f"Reorg(new_head={self.new_head})"


@dataclass
class NewEvents:
    block_hash: bytes
    block_number: int
    events: List[Event]

    @staticmethod
    def from_proto(p: indexer_service_pb2.NewEvents):
        events = [Event.from_proto(ev) for ev in p.events]
        return NewEvents(bytes(p.block_hash), p.block_number, events)

    def __str__(self) -> str:
        return f"NewEvents(block_hash=0x{self.block_hash.hex()}, block_number={self.block_number}, ...{len(self.events)} events)"



"""Apibara indexer runner."""

from contextlib import contextmanager
from dataclasses import dataclass
from typing import Awaitable, Callable, Generic, List, Optional, TypeVar

import backoff
from grpc.aio import AioRpcError

from apibara.client import Client
from apibara.indexer.indexer import IndexerClient
from apibara.indexer.storage import IndexerStorage, Storage
from apibara.logging import logger
from apibara.model import EventFilter, Indexer, NewBlock, NewEvents, Reorg
from apibara.rpc import RpcClient
from apibara.starknet import get_selector_from_name
from apibara.starknet.rpc import StarkNetRpcClient

UserContext = TypeVar("UserContext")


@dataclass
class Info(Generic[UserContext]):
    context: UserContext
    rpc_client: RpcClient
    storage: Storage


NewEventsHandler = Callable[[Info, NewEvents], Awaitable[None]]
BlockHandler = Callable[[Info, NewBlock], Awaitable[None]]
ReorgHandler = Callable[[Info, Reorg], Awaitable[None]]


@dataclass
class IndexerRunnerConfiguration:
    apibara_url: Optional[str] = None
    rpc_url: Optional[str] = None
    storage_url: Optional[str] = None


class IndexerRunner(Generic[UserContext]):
    """Run an indexer, listening for new events and calling the provided callbacks."""

    def __init__(
        self,
        *,
        indexer_id: str,
        new_events_handler: NewEventsHandler,
        config: Optional[IndexerRunnerConfiguration] = None,
    ) -> None:
        if config is None:
            config = IndexerRunnerConfiguration()
        self._indexer_id = indexer_id
        self._config = config
        self._new_events_handler = new_events_handler
        self._block_handler = None
        self._reorg_handler = None
        self._context = None
        self._indexer_storage = IndexerStorage(
            self._config.storage_url, self._indexer_id
        )
        self._rpc_client = StarkNetRpcClient(self._config.rpc_url)

        self._indexer_config = None
        self._event_topic_to_name_map = None

    def create_if_not_exists(
        self,
        filters: Optional[List[EventFilter]] = None,
        index_from_block: Optional[int] = None,
    ):
        """If the indexer does not exist on the server, create it."""
        self._indexer_config = {
            "filters": filters,
            "index_from_block": index_from_block,
        }

    def set_context(self, context: UserContext):
        """Set the context object used to share information between handlers."""
        self._context = context

    def add_block_handler(self, block_handler: BlockHandler) -> None:
        """Add a callback called every time there is a new block."""
        self._block_handler = block_handler

    def add_reorg_handler(self, reorg_handler: ReorgHandler) -> None:
        """Add a callback called every time there is a chain reorganization."""
        self._reorg_handler = reorg_handler

    async def run(self):
        """Run the indexer until stopped."""
        # Call the helper function that will retry in case of GrpError,
        # like for example on disconnect.
        await self._do_run()

    @backoff.on_exception(backoff.expo, AioRpcError, logger=logger)
    async def _do_run(self):
        async with Client.connect(self._config.apibara_url) as client:
            logger.debug("connected to apibara server")
            indexer_client = client.indexer_client()
            existing = await indexer_client.get_indexer(self._indexer_id)
            if existing is None:
                indexer = await self._maybe_create_indexer(indexer_client)
            else:
                indexer = existing

            self._initialize_event_topic_map()

            await self._run_indexer(indexer_client, indexer)

    async def _run_indexer(self, client: IndexerClient, indexer: Indexer):
        async with client.connect(indexer) as stream:
            async for message in stream:
                if isinstance(message, NewBlock):
                    await self._handle_new_block(message)
                elif isinstance(message, Reorg):
                    await self._handle_reorg(message)
                elif isinstance(message, NewEvents):
                    await self._handle_new_events(message)
                    # inform server that events were handled
                    await stream.ack_block(message.block_hash)
                    logger.debug(f"Acked block 0x{message.block_hash.hex()}")
                else:
                    raise RuntimeError(f"Unknown message: {message}")

    @contextmanager
    def _block_context(self, number: int) -> Info[UserContext]:
        with self._indexer_storage.create_storage_for_block(number) as storage:
            yield Info(
                context=self._context, rpc_client=self._rpc_client, storage=storage
            )

    async def _handle_new_block(self, message: NewBlock):
        if self._block_handler is None:
            return
        with self._block_context(message.new_head.number) as info:
            await self._block_handler(info, message)

    async def _handle_reorg(self, message: Reorg):
        if self._reorg_handler is None:
            return
        # TODO: invalidate old data. Before or after user's callback?
        # Why not two callbacks?
        with self._block_context(message.new_head.number) as info:
            await self._reorg_handler(info, message)

    async def _handle_new_events(self, message: NewEvents):
        assert self._new_events_handler is not None
        # Add event name to events
        for event in message.events:
            if len(event.topics) == 1:
                topic = event.topics[0].lstrip(b"\x00")
                name = self._event_topic_to_name_map.get(topic)
                event.name = name
        with self._block_context(message.block_number) as info:
            await self._new_events_handler(info, message)

    async def _maybe_create_indexer(self, indexer_client: IndexerClient) -> Indexer:
        if self._indexer_config is None:
            raise RuntimeError(f"Indexer {self._indexer_id} does not exist")
        index_from_block = self._indexer_config["index_from_block"]
        filters = self._indexer_config["filters"]
        indexer = await indexer_client.create_indexer(
            self._indexer_id, index_from_block, filters
        )
        logger.debug("Created new indexer")
        return indexer

    def _initialize_event_topic_map(self):
        self._event_topic_to_name_map = dict()
        if self._indexer_config is None:
            return
        filters = self._indexer_config["filters"]
        if filters is None:
            return

        result = dict()
        for filter in filters:
            if filter.name is None:
                continue
            topic_hex = hex(get_selector_from_name(filter.name))[2:]
            # fromhex requires an even number of digits
            if len(topic_hex) % 2 == 1:
                topic_hex = "0" + topic_hex
            topic_value = bytes.fromhex(topic_hex).lstrip(b"\x00")
            result[topic_value] = filter.name

        self._event_topic_to_name_map = result



__version__ = "0.3.0"

from .client import Client
from .indexer.indexer import IndexerClient, IndexerStream
from .indexer.runner import IndexerRunner, Info
from .model import Indexer, IndexerConnected, NewBlock, NewEvents, Reorg



from contextlib import contextmanager
from typing import Any, Iterable, Iterator, Optional

from pymongo import MongoClient
from pymongo.database import Database

Document = dict[str, Any]
Filter = dict[str, Any]
Update = dict[str, Any]
Projection = dict[str, any]


class IndexerStorage:
    """Create instances of Storage."""

    def __init__(self, url: Optional[str], indexer_id: str) -> None:
        if url is None:
            raise ValueError("Storage url must be not None")

        self.db_name = indexer_id.replace("-", "_")

        self._mongo = MongoClient(url)
        self._db = self._mongo[self.db_name]

    @contextmanager
    def create_storage_for_block(self, block_number: int) -> Iterator["Storage"]:
        yield Storage(self._db, block_number)


class Storage:
    # NOTE: all methods are async even though it's using the sync mongo client.
    # In the near future the API will connect to mongo through Apibara Server and
    # so it will be async.
    """Chain-aware document storage."""

    def __init__(self, db: Database, block_number: int) -> None:
        self._db = db
        self._block_number = block_number

    async def insert_one(self, collection: str, doc: Document):
        self._add_chain_information(doc)
        self._db[collection].insert_one(doc)

    async def insert_many(self, collection: str, docs: Iterable[Document]):
        for doc in docs:
            self._add_chain_information(doc)
        self._db[collection].insert_many(docs)

    async def delete_one(self, collection: str, filter: Filter):
        self._db[collection].update_one(
            filter, {"$set": {"_chain.valid_to": self._block_number}}
        )

    async def delete_many(self, collection: str, filter: Filter):
        self._db[collection].update_many(
            filter, {"$set": {"_chain.valid_to": self._block_number}}
        )

    async def find_one(self, collection: str, filter: Filter) -> Optional[Document]:
        self._add_current_block_to_filter(filter)
        return self._db[collection].find_one(filter)

    async def find(
        self,
        collection: str,
        filter: Filter,
        projection: Optional[Projection] = None,
        skip: Optional[int] = None,
        limit: Optional[int] = None,
    ) -> Iterator[dict]:
        self._add_current_block_to_filter(filter)
        return self._db[collection].find(filter, projection, skip, limit)

    async def find_one_and_replace(
        self,
        collection: str,
        filter: Filter,
        replacement: Document,
        upsert: bool = False,
    ):
        # Step 1. Update the old document (if any) by clamping its validity range
        self._add_current_block_to_filter(filter)
        existing = self._db[collection].find_one_and_update(
            filter, {"$set": {"_chain.valid_to": self._block_number}}
        )

        # Step 2. Insert the new document.
        # Insert only if the existing document exists or if upsert.
        if existing is not None or upsert:
            await self.insert_one(collection, replacement)

        return existing

    async def find_one_and_update(
        self, collection: str, filter: Filter, update: Update
    ):
        # Step 1. Update the old document (if any) by clamping its validity range
        self._add_current_block_to_filter(filter)
        existing = self._db[collection].find_one_and_update(
            filter, {"$set": {"_chain.valid_to": self._block_number}}
        )

        # Step 2. To simulate an update, first insert then call update on it.
        if existing is not None:
            del existing["_id"]
            del existing["_chain"]
            await self.insert_one(collection, existing)
            self._db[collection].update_one(filter, update)

        return existing

    def _add_chain_information(self, doc: Document):
        doc["_chain"] = {"valid_from": self._block_number, "valid_to": None}

    def _add_current_block_to_filter(self, filter: Filter):
        filter["_chain.valid_to"] = None



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: apibara/application/indexer_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import \
    timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n)apibara/application/indexer_service.proto\x12\x1c\x61pibara.application.v1alpha1\x1a\x1fgoogle/protobuf/timestamp.proto"S\n\x0b\x45ventFilter\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\x0c\x12\x33\n\x06topics\x18\x02 \x03(\x0b\x32#.apibara.application.v1alpha1.Topic"B\n\x05Topic\x12\x39\n\x07\x63hoices\x18\x01 \x03(\x0b\x32(.apibara.application.v1alpha1.TopicValue"\x1b\n\nTopicValue\x12\r\n\x05value\x18\x01 \x01(\x0c"\x9f\x01\n\x07Indexer\x12\n\n\x02id\x18\x01 \x01(\t\x12\x1d\n\x10indexed_to_block\x18\x02 \x01(\x04H\x00\x88\x01\x01\x12\x18\n\x10index_from_block\x18\x03 \x01(\x04\x12:\n\x07\x66ilters\x18\x04 \x03(\x0b\x32).apibara.application.v1alpha1.EventFilterB\x13\n\x11_indexed_to_block"x\n\x14\x43reateIndexerRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x18\n\x10index_from_block\x18\x02 \x01(\x04\x12:\n\x07\x66ilters\x18\x03 \x03(\x0b\x32).apibara.application.v1alpha1.EventFilter"O\n\x15\x43reateIndexerResponse\x12\x36\n\x07indexer\x18\x01 \x01(\x0b\x32%.apibara.application.v1alpha1.Indexer"\x1f\n\x11GetIndexerRequest\x12\n\n\x02id\x18\x01 \x01(\t"L\n\x12GetIndexerResponse\x12\x36\n\x07indexer\x18\x01 \x01(\x0b\x32%.apibara.application.v1alpha1.Indexer"\x14\n\x12ListIndexerRequest"N\n\x13ListIndexerResponse\x12\x37\n\x08indexers\x18\x01 \x03(\x0b\x32%.apibara.application.v1alpha1.Indexer""\n\x14\x44\x65leteIndexerRequest\x12\n\n\x02id\x18\x01 \x01(\t"O\n\x15\x44\x65leteIndexerResponse\x12\x36\n\x07indexer\x18\x01 \x01(\x0b\x32%.apibara.application.v1alpha1.Indexer"\x84\x01\n\x0b\x42lockHeader\x12\x0c\n\x04hash\x18\x01 \x01(\x0c\x12\x18\n\x0bparent_hash\x18\x02 \x01(\x0cH\x00\x88\x01\x01\x12\x0e\n\x06number\x18\x03 \x01(\x04\x12-\n\ttimestamp\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x0e\n\x0c_parent_hash"\x9f\x01\n\x05\x45vent\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\x0c\x12\x13\n\x0b\x62lock_index\x18\x02 \x01(\x04\x12\x38\n\x06topics\x18\x03 \x03(\x0b\x32(.apibara.application.v1alpha1.TopicValue\x12\x36\n\x04\x64\x61ta\x18\x04 \x03(\x0b\x32(.apibara.application.v1alpha1.TopicValue"\x1c\n\x0e\x43onnectIndexer\x12\n\n\x02id\x18\x01 \x01(\t"J\n\x10IndexerConnected\x12\x36\n\x07indexer\x18\x01 \x01(\x0b\x32%.apibara.application.v1alpha1.Indexer"\x18\n\x08\x41\x63kBlock\x12\x0c\n\x04hash\x18\x01 \x01(\x0c"\x9a\x01\n\x15\x43onnectIndexerRequest\x12?\n\x07\x63onnect\x18\x01 \x01(\x0b\x32,.apibara.application.v1alpha1.ConnectIndexerH\x00\x12\x35\n\x03\x61\x63k\x18\x03 \x01(\x0b\x32&.apibara.application.v1alpha1.AckBlockH\x00\x42\t\n\x07message"G\n\x08NewBlock\x12;\n\x08new_head\x18\x01 \x01(\x0b\x32).apibara.application.v1alpha1.BlockHeader"D\n\x05Reorg\x12;\n\x08new_head\x18\x01 \x01(\x0b\x32).apibara.application.v1alpha1.BlockHeader"j\n\tNewEvents\x12\x12\n\nblock_hash\x18\x01 \x01(\x0c\x12\x14\n\x0c\x62lock_number\x18\x02 \x01(\x04\x12\x33\n\x06\x65vents\x18\x03 \x03(\x0b\x32#.apibara.application.v1alpha1.Event"\x9a\x02\n\x16\x43onnectIndexerResponse\x12\x43\n\tconnected\x18\x01 \x01(\x0b\x32..apibara.application.v1alpha1.IndexerConnectedH\x00\x12;\n\tnew_block\x18\x02 \x01(\x0b\x32&.apibara.application.v1alpha1.NewBlockH\x00\x12\x34\n\x05reorg\x18\x03 \x01(\x0b\x32#.apibara.application.v1alpha1.ReorgH\x00\x12=\n\nnew_events\x18\x04 \x01(\x0b\x32\'.apibara.application.v1alpha1.NewEventsH\x00\x42\t\n\x07message2\xea\x04\n\x0eIndexerManager\x12x\n\rCreateIndexer\x12\x32.apibara.application.v1alpha1.CreateIndexerRequest\x1a\x33.apibara.application.v1alpha1.CreateIndexerResponse\x12o\n\nGetIndexer\x12/.apibara.application.v1alpha1.GetIndexerRequest\x1a\x30.apibara.application.v1alpha1.GetIndexerResponse\x12r\n\x0bListIndexer\x12\x30.apibara.application.v1alpha1.ListIndexerRequest\x1a\x31.apibara.application.v1alpha1.ListIndexerResponse\x12x\n\rDeleteIndexer\x12\x32.apibara.application.v1alpha1.DeleteIndexerRequest\x1a\x33.apibara.application.v1alpha1.DeleteIndexerResponse\x12\x7f\n\x0e\x43onnectIndexer\x12\x33.apibara.application.v1alpha1.ConnectIndexerRequest\x1a\x34.apibara.application.v1alpha1.ConnectIndexerResponse(\x01\x30\x01\x62\x06proto3'
)


_EVENTFILTER = DESCRIPTOR.message_types_by_name["EventFilter"]
_TOPIC = DESCRIPTOR.message_types_by_name["Topic"]
_TOPICVALUE = DESCRIPTOR.message_types_by_name["TopicValue"]
_INDEXER = DESCRIPTOR.message_types_by_name["Indexer"]
_CREATEINDEXERREQUEST = DESCRIPTOR.message_types_by_name["CreateIndexerRequest"]
_CREATEINDEXERRESPONSE = DESCRIPTOR.message_types_by_name["CreateIndexerResponse"]
_GETINDEXERREQUEST = DESCRIPTOR.message_types_by_name["GetIndexerRequest"]
_GETINDEXERRESPONSE = DESCRIPTOR.message_types_by_name["GetIndexerResponse"]
_LISTINDEXERREQUEST = DESCRIPTOR.message_types_by_name["ListIndexerRequest"]
_LISTINDEXERRESPONSE = DESCRIPTOR.message_types_by_name["ListIndexerResponse"]
_DELETEINDEXERREQUEST = DESCRIPTOR.message_types_by_name["DeleteIndexerRequest"]
_DELETEINDEXERRESPONSE = DESCRIPTOR.message_types_by_name["DeleteIndexerResponse"]
_BLOCKHEADER = DESCRIPTOR.message_types_by_name["BlockHeader"]
_EVENT = DESCRIPTOR.message_types_by_name["Event"]
_CONNECTINDEXER = DESCRIPTOR.message_types_by_name["ConnectIndexer"]
_INDEXERCONNECTED = DESCRIPTOR.message_types_by_name["IndexerConnected"]
_ACKBLOCK = DESCRIPTOR.message_types_by_name["AckBlock"]
_CONNECTINDEXERREQUEST = DESCRIPTOR.message_types_by_name["ConnectIndexerRequest"]
_NEWBLOCK = DESCRIPTOR.message_types_by_name["NewBlock"]
_REORG = DESCRIPTOR.message_types_by_name["Reorg"]
_NEWEVENTS = DESCRIPTOR.message_types_by_name["NewEvents"]
_CONNECTINDEXERRESPONSE = DESCRIPTOR.message_types_by_name["ConnectIndexerResponse"]
EventFilter = _reflection.GeneratedProtocolMessageType(
    "EventFilter",
    (_message.Message,),
    {
        "DESCRIPTOR": _EVENTFILTER,
        "__module__": "apibara.application.indexer_service_pb2"
        # @@protoc_insertion_point(class_scope:apibara.application.v1alpha1.EventFilter)
    },
)
_sym_db.RegisterMessage(EventFilter)

Topic = _reflection.GeneratedProtocolMessageType(
    "Topic",
    (_message.Message,),
    {
        "DESCRIPTOR": _TOPIC,
        "__module__": "apibara.application.indexer_service_pb2"
        # @@protoc_insertion_point(class_scope:apibara.application.v1alpha1.Topic)
    },
)
_sym_db.RegisterMessage(Topic)

TopicValue = _reflection.GeneratedProtocolMessageType(
    "TopicValue",
    (_message.Message,),
    {
        "DESCRIPTOR": _TOPICVALUE,
        "__module__": "apibara.application.indexer_service_pb2"
        # @@protoc_insertion_point(class_scope:apibara.application.v1alpha1.TopicValue)
    },
)
_sym_db.RegisterMessage(TopicValue)

Indexer = _reflection.GeneratedProtocolMessageType(
    "Indexer",
    (_message.Message,),
    {
        "DESCRIPTOR": _INDEXER,
        "__module__": "apibara.application.indexer_service_pb2"
        # @@protoc_insertion_point(class_scope:apibara.application.v1alpha1.Indexer)
    },
)
_sym_db.RegisterMessage(Indexer)

CreateIndexerRequest = _reflection.GeneratedProtocolMessageType(
    "CreateIndexerRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _CREATEINDEXERREQUEST,
        "__module__": "apibara.application.indexer_service_pb2"
        # @@protoc_insertion_point(class_scope:apibara.application.v1alpha1.CreateIndexerRequest)
    },
)
_sym_db.RegisterMessage(CreateIndexerRequest)

CreateIndexerResponse = _reflection.GeneratedProtocolMessageType(
    "CreateIndexerResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _CREATEINDEXERRESPONSE,
        "__module__": "apibara.application.indexer_service_pb2"
        # @@protoc_insertion_point(class_scope:apibara.application.v1alpha1.CreateIndexerResponse)
    },
)
_sym_db.RegisterMessage(CreateIndexerResponse)

GetIndexerRequest = _reflection.GeneratedProtocolMessageType(
    "GetIndexerRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETINDEXERREQUEST,
        "__module__": "apibara.application.indexer_service_pb2"
        # @@protoc_insertion_point(class_scope:apibara.application.v1alpha1.GetIndexerRequest)
    },
)
_sym_db.RegisterMessage(GetIndexerRequest)

GetIndexerResponse = _reflection.GeneratedProtocolMessageType(
    "GetIndexerResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETINDEXERRESPONSE,
        "__module__": "apibara.application.indexer_service_pb2"
        # @@protoc_insertion_point(class_scope:apibara.application.v1alpha1.GetIndexerResponse)
    },
)
_sym_db.RegisterMessage(GetIndexerResponse)

ListIndexerRequest = _reflection.GeneratedProtocolMessageType(
    "ListIndexerRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _LISTINDEXERREQUEST,
        "__module__": "apibara.application.indexer_service_pb2"
        # @@protoc_insertion_point(class_scope:apibara.application.v1alpha1.ListIndexerRequest)
    },
)
_sym_db.RegisterMessage(ListIndexerRequest)

ListIndexerResponse = _reflection.GeneratedProtocolMessageType(
    "ListIndexerResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _LISTINDEXERRESPONSE,
        "__module__": "apibara.application.indexer_service_pb2"
        # @@protoc_insertion_point(class_scope:apibara.application.v1alpha1.ListIndexerResponse)
    },
)
_sym_db.RegisterMessage(ListIndexerResponse)

DeleteIndexerRequest = _reflection.GeneratedProtocolMessageType(
    "DeleteIndexerRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _DELETEINDEXERREQUEST,
        "__module__": "apibara.application.indexer_service_pb2"
        # @@protoc_insertion_point(class_scope:apibara.application.v1alpha1.DeleteIndexerRequest)
    },
)
_sym_db.RegisterMessage(DeleteIndexerRequest)

DeleteIndexerResponse = _reflection.GeneratedProtocolMessageType(
    "DeleteIndexerResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _DELETEINDEXERRESPONSE,
        "__module__": "apibara.application.indexer_service_pb2"
        # @@protoc_insertion_point(class_scope:apibara.application.v1alpha1.DeleteIndexerResponse)
    },
)
_sym_db.RegisterMessage(DeleteIndexerResponse)

BlockHeader = _reflection.GeneratedProtocolMessageType(
    "BlockHeader",
    (_message.Message,),
    {
        "DESCRIPTOR": _BLOCKHEADER,
        "__module__": "apibara.application.indexer_service_pb2"
        # @@protoc_insertion_point(class_scope:apibara.application.v1alpha1.BlockHeader)
    },
)
_sym_db.RegisterMessage(BlockHeader)

Event = _reflection.GeneratedProtocolMessageType(
    "Event",
    (_message.Message,),
    {
        "DESCRIPTOR": _EVENT,
        "__module__": "apibara.application.indexer_service_pb2"
        # @@protoc_insertion_point(class_scope:apibara.application.v1alpha1.Event)
    },
)
_sym_db.RegisterMessage(Event)

ConnectIndexer = _reflection.GeneratedProtocolMessageType(
    "ConnectIndexer",
    (_message.Message,),
    {
        "DESCRIPTOR": _CONNECTINDEXER,
        "__module__": "apibara.application.indexer_service_pb2"
        # @@protoc_insertion_point(class_scope:apibara.application.v1alpha1.ConnectIndexer)
    },
)
_sym_db.RegisterMessage(ConnectIndexer)

IndexerConnected = _reflection.GeneratedProtocolMessageType(
    "IndexerConnected",
    (_message.Message,),
    {
        "DESCRIPTOR": _INDEXERCONNECTED,
        "__module__": "apibara.application.indexer_service_pb2"
        # @@protoc_insertion_point(class_scope:apibara.application.v1alpha1.IndexerConnected)
    },
)
_sym_db.RegisterMessage(IndexerConnected)

AckBlock = _reflection.GeneratedProtocolMessageType(
    "AckBlock",
    (_message.Message,),
    {
        "DESCRIPTOR": _ACKBLOCK,
        "__module__": "apibara.application.indexer_service_pb2"
        # @@protoc_insertion_point(class_scope:apibara.application.v1alpha1.AckBlock)
    },
)
_sym_db.RegisterMessage(AckBlock)

ConnectIndexerRequest = _reflection.GeneratedProtocolMessageType(
    "ConnectIndexerRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _CONNECTINDEXERREQUEST,
        "__module__": "apibara.application.indexer_service_pb2"
        # @@protoc_insertion_point(class_scope:apibara.application.v1alpha1.ConnectIndexerRequest)
    },
)
_sym_db.RegisterMessage(ConnectIndexerRequest)

NewBlock = _reflection.GeneratedProtocolMessageType(
    "NewBlock",
    (_message.Message,),
    {
        "DESCRIPTOR": _NEWBLOCK,
        "__module__": "apibara.application.indexer_service_pb2"
        # @@protoc_insertion_point(class_scope:apibara.application.v1alpha1.NewBlock)
    },
)
_sym_db.RegisterMessage(NewBlock)

Reorg = _reflection.GeneratedProtocolMessageType(
    "Reorg",
    (_message.Message,),
    {
        "DESCRIPTOR": _REORG,
        "__module__": "apibara.application.indexer_service_pb2"
        # @@protoc_insertion_point(class_scope:apibara.application.v1alpha1.Reorg)
    },
)
_sym_db.RegisterMessage(Reorg)

NewEvents = _reflection.GeneratedProtocolMessageType(
    "NewEvents",
    (_message.Message,),
    {
        "DESCRIPTOR": _NEWEVENTS,
        "__module__": "apibara.application.indexer_service_pb2"
        # @@protoc_insertion_point(class_scope:apibara.application.v1alpha1.NewEvents)
    },
)
_sym_db.RegisterMessage(NewEvents)

ConnectIndexerResponse = _reflection.GeneratedProtocolMessageType(
    "ConnectIndexerResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _CONNECTINDEXERRESPONSE,
        "__module__": "apibara.application.indexer_service_pb2"
        # @@protoc_insertion_point(class_scope:apibara.application.v1alpha1.ConnectIndexerResponse)
    },
)
_sym_db.RegisterMessage(ConnectIndexerResponse)

_INDEXERMANAGER = DESCRIPTOR.services_by_name["IndexerManager"]
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    _EVENTFILTER._serialized_start = 108
    _EVENTFILTER._serialized_end = 191
    _TOPIC._serialized_start = 193
    _TOPIC._serialized_end = 259
    _TOPICVALUE._serialized_start = 261
    _TOPICVALUE._serialized_end = 288
    _INDEXER._serialized_start = 291
    _INDEXER._serialized_end = 450
    _CREATEINDEXERREQUEST._serialized_start = 452
    _CREATEINDEXERREQUEST._serialized_end = 572
    _CREATEINDEXERRESPONSE._serialized_start = 574
    _CREATEINDEXERRESPONSE._serialized_end = 653
    _GETINDEXERREQUEST._serialized_start = 655
    _GETINDEXERREQUEST._serialized_end = 686
    _GETINDEXERRESPONSE._serialized_start = 688
    _GETINDEXERRESPONSE._serialized_end = 764
    _LISTINDEXERREQUEST._serialized_start = 766
    _LISTINDEXERREQUEST._serialized_end = 786
    _LISTINDEXERRESPONSE._serialized_start = 788
    _LISTINDEXERRESPONSE._serialized_end = 866
    _DELETEINDEXERREQUEST._serialized_start = 868
    _DELETEINDEXERREQUEST._serialized_end = 902
    _DELETEINDEXERRESPONSE._serialized_start = 904
    _DELETEINDEXERRESPONSE._serialized_end = 983
    _BLOCKHEADER._serialized_start = 986
    _BLOCKHEADER._serialized_end = 1118
    _EVENT._serialized_start = 1121
    _EVENT._serialized_end = 1280
    _CONNECTINDEXER._serialized_start = 1282
    _CONNECTINDEXER._serialized_end = 1310
    _INDEXERCONNECTED._serialized_start = 1312
    _INDEXERCONNECTED._serialized_end = 1386
    _ACKBLOCK._serialized_start = 1388
    _ACKBLOCK._serialized_end = 1412
    _CONNECTINDEXERREQUEST._serialized_start = 1415
    _CONNECTINDEXERREQUEST._serialized_end = 1569
    _NEWBLOCK._serialized_start = 1571
    _NEWBLOCK._serialized_end = 1642
    _REORG._serialized_start = 1644
    _REORG._serialized_end = 1712
    _NEWEVENTS._serialized_start = 1714
    _NEWEVENTS._serialized_end = 1820
    _CONNECTINDEXERRESPONSE._serialized_start = 1823
    _CONNECTINDEXERRESPONSE._serialized_end = 2105
    _INDEXERMANAGER._serialized_start = 2108
    _INDEXERMANAGER._serialized_end = 2726
# @@protoc_insertion_point(module_scope)



from typing import Any, List, Optional

import aiohttp

from apibara.rpc import RpcClient
from apibara.starknet.hash import get_selector_from_name


class StarkNetRpcClient(RpcClient):
    def __init__(self, url: Optional[str]) -> None:
        if url is None:
            url = "https://starknet-goerli.apibara.com"
        self._url = url

    async def _request(self, method: str, params: List[Any]):
        async with aiohttp.ClientSession(self._url) as session:
            data = {"id": 1, "jsonrpc": "2.0", "method": method, "params": params}
            async with session.post("/", json=data) as response:
                response = await response.json()
                if "result" in response:
                    return response["result"]
                raise RuntimeError(response["error"]["message"])

    async def get_block_by_hash(self, hash: bytes) -> dict:
        return await self._request(
            "starknet_getBlockByHash", ["0x" + hash.hex(), "TXN_HASH"]
        )

    async def get_block_by_number(self, number: int) -> dict:
        return await self._request(
            "starknet_getBlockByNumber", [hex(number), "TXN_HASH"]
        )

    async def call(self, address: bytes, method: str, params: List[Any]) -> dict:
        params = [
            {
                "contract_address": "0x" + address.hex(),
                "entry_point_selector": hex(get_selector_from_name(method)),
                "calldata": params,
            },
            "latest",
        ]
        return await self._request("starknet_call", params)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from apibara.application import \
    indexer_service_pb2 as apibara_dot_application_dot_indexer__service__pb2


class IndexerManagerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.CreateIndexer = channel.unary_unary(
            "/apibara.application.v1alpha1.IndexerManager/CreateIndexer",
            request_serializer=apibara_dot_application_dot_indexer__service__pb2.CreateIndexerRequest.SerializeToString,
            response_deserializer=apibara_dot_application_dot_indexer__service__pb2.CreateIndexerResponse.FromString,
        )
        self.GetIndexer = channel.unary_unary(
            "/apibara.application.v1alpha1.IndexerManager/GetIndexer",
            request_serializer=apibara_dot_application_dot_indexer__service__pb2.GetIndexerRequest.SerializeToString,
            response_deserializer=apibara_dot_application_dot_indexer__service__pb2.GetIndexerResponse.FromString,
        )
        self.ListIndexer = channel.unary_unary(
            "/apibara.application.v1alpha1.IndexerManager/ListIndexer",
            request_serializer=apibara_dot_application_dot_indexer__service__pb2.ListIndexerRequest.SerializeToString,
            response_deserializer=apibara_dot_application_dot_indexer__service__pb2.ListIndexerResponse.FromString,
        )
        self.DeleteIndexer = channel.unary_unary(
            "/apibara.application.v1alpha1.IndexerManager/DeleteIndexer",
            request_serializer=apibara_dot_application_dot_indexer__service__pb2.DeleteIndexerRequest.SerializeToString,
            response_deserializer=apibara_dot_application_dot_indexer__service__pb2.DeleteIndexerResponse.FromString,
        )
        self.ConnectIndexer = channel.stream_stream(
            "/apibara.application.v1alpha1.IndexerManager/ConnectIndexer",
            request_serializer=apibara_dot_application_dot_indexer__service__pb2.ConnectIndexerRequest.SerializeToString,
            response_deserializer=apibara_dot_application_dot_indexer__service__pb2.ConnectIndexerResponse.FromString,
        )


class IndexerManagerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def CreateIndexer(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetIndexer(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListIndexer(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DeleteIndexer(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ConnectIndexer(self, request_iterator, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_IndexerManagerServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "CreateIndexer": grpc.unary_unary_rpc_method_handler(
            servicer.CreateIndexer,
            request_deserializer=apibara_dot_application_dot_indexer__service__pb2.CreateIndexerRequest.FromString,
            response_serializer=apibara_dot_application_dot_indexer__service__pb2.CreateIndexerResponse.SerializeToString,
        ),
        "GetIndexer": grpc.unary_unary_rpc_method_handler(
            servicer.GetIndexer,
            request_deserializer=apibara_dot_application_dot_indexer__service__pb2.GetIndexerRequest.FromString,
            response_serializer=apibara_dot_application_dot_indexer__service__pb2.GetIndexerResponse.SerializeToString,
        ),
        "ListIndexer": grpc.unary_unary_rpc_method_handler(
            servicer.ListIndexer,
            request_deserializer=apibara_dot_application_dot_indexer__service__pb2.ListIndexerRequest.FromString,
            response_serializer=apibara_dot_application_dot_indexer__service__pb2.ListIndexerResponse.SerializeToString,
        ),
        "DeleteIndexer": grpc.unary_unary_rpc_method_handler(
            servicer.DeleteIndexer,
            request_deserializer=apibara_dot_application_dot_indexer__service__pb2.DeleteIndexerRequest.FromString,
            response_serializer=apibara_dot_application_dot_indexer__service__pb2.DeleteIndexerResponse.SerializeToString,
        ),
        "ConnectIndexer": grpc.stream_stream_rpc_method_handler(
            servicer.ConnectIndexer,
            request_deserializer=apibara_dot_application_dot_indexer__service__pb2.ConnectIndexerRequest.FromString,
            response_serializer=apibara_dot_application_dot_indexer__service__pb2.ConnectIndexerResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "apibara.application.v1alpha1.IndexerManager", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class IndexerManager(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def CreateIndexer(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/apibara.application.v1alpha1.IndexerManager/CreateIndexer",
            apibara_dot_application_dot_indexer__service__pb2.CreateIndexerRequest.SerializeToString,
            apibara_dot_application_dot_indexer__service__pb2.CreateIndexerResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def GetIndexer(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/apibara.application.v1alpha1.IndexerManager/GetIndexer",
            apibara_dot_application_dot_indexer__service__pb2.GetIndexerRequest.SerializeToString,
            apibara_dot_application_dot_indexer__service__pb2.GetIndexerResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ListIndexer(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/apibara.application.v1alpha1.IndexerManager/ListIndexer",
            apibara_dot_application_dot_indexer__service__pb2.ListIndexerRequest.SerializeToString,
            apibara_dot_application_dot_indexer__service__pb2.ListIndexerResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def DeleteIndexer(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/apibara.application.v1alpha1.IndexerManager/DeleteIndexer",
            apibara_dot_application_dot_indexer__service__pb2.DeleteIndexerRequest.SerializeToString,
            apibara_dot_application_dot_indexer__service__pb2.DeleteIndexerResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ConnectIndexer(
        request_iterator,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.stream_stream(
            request_iterator,
            target,
            "/apibara.application.v1alpha1.IndexerManager/ConnectIndexer",
            apibara_dot_application_dot_indexer__service__pb2.ConnectIndexerRequest.SerializeToString,
            apibara_dot_application_dot_indexer__service__pb2.ConnectIndexerResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['apibara', 'apibara.application', 'apibara.indexer', 'apibara.starknet']

package_data = \
{'': ['*']}

install_requires = \
['aiochannel>=1.1.1,<2.0.0',
 'aiohttp>=3.8.1,<4.0.0',
 'backoff>=2.1.2,<3.0.0',
 'click-help-colors>=0.9.1,<0.10.0',
 'click>=8.1.3,<9.0.0',
 'eth-hash[pysha3]>=0.3.2,<0.4.0',
 'grpcio-tools>=1.47.0,<2.0.0',
 'grpcio>=1.47.0,<2.0.0',
 'pymongo[srv]>=4.1.1,<5.0.0']

entry_points = \
{'console_scripts': ['apibara = apibara.cli:cli']}

setup_kwargs = {
    'name': 'apibara',
    'version': '0.3.0',
    'description': 'Apibara cliend SDK. Build web3-powered applications.',
    'long_description': 'Apibara Python SDK\n==================\n\n.. warning::\n    This SDK is alpha software. The API will change drastically until the beta.\n\n    `Open an issue on GitHub <https://github.com/apibara/python-sdk>`_ to report bugs or provide feedback.\n\n\nBuild web3-powered applications in Python. \n\n`Apibara <https://github.com/apibara/apibara>`_ is a tool to build web APIs that\nintegrate data from one or more blockchains. Simply define a new indexer service\nand the Apibara service will send it historical and live events to index.\n\n\nGetting started\n---------------\n\nStart by running MongoDB and Apibara using the `docker-compose.yml` file included in `examples/starknet`.\n\n.. code::\n\n    docker-compose up\n\n\nThen you can run the example script.\n\n.. code::\n\n    python examples/starknet/main.py\n\n\nLicense\n-------\n\n   Copyright 2022 GNC Labs Limited\n\n   Licensed under the Apache License, Version 2.0 (the "License");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an "AS IS" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.\n',
    'author': 'Francesco Ceccon',
    'author_email': 'francesco@apibara.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://www.apibara.com',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7.2,<3.10',
}


setup(**setup_kwargs)
